/*
 * Defines a World node.
 * Inherits from Node.
 */

#ifndef __RC_WORLD_H__
#define __RC_WORLD_H__

class World : public Node {
	friend class SceneGraph;

/*----------------------------------------------------------[ Functions ]----*/

	protected:
		World(
				const char		*nname
			);

		~World();

	public:
		/* Recursively draw the entire scene */
		void drawAll(
				DeviceState_t	*device_state
			);

		/*
		 * A camera must be set as the active camera, so that the scene is
		 * rendered from a specified point, with the cameras projection
		 * settings.
		 */
		void setActiveCamera(
				Camera		*camera
			);

		Camera *getActiveCamera();

		/* Clear the color buffer with color */
		void setClearColorBuffer(
				bool		do_clear,
				rgbaf		color
			);

		/* Clear the depth buffer with a depth value */
		void setClearDepthBuffer(
				bool		do_clear,
				f32			depth
			);

		/* Clear the stencil buffer with a stencil value */
		void setClearStencilBuffer(
				bool		do_clear,
				i32			stencil
			);

		RenderState *getStandardState();

		/* Set viewport (pixels) */
		void setViewport(
				i32			x,
				i32			y,
				u32			width,
				u32			height
			);

		/* Reset the viewport to fullscreen (default) */
		void setFullViewport();

	private:
		void transformCamera(
				mat4f *matrix
			);

		Node *duplicateSelf();

/*------------------------------------------------------------[ Members ]----*/

	private:
		Camera			*activeCamera;
		RenderState		standardState;
		
		rgbaf			clearColor;
		f32				clearDepth;
		i32				clearStencil;

		bool			clearColorBuffer;
		bool			clearDepthBuffer;
		bool			clearStencilBuffer;

		bool			full_viewport;
		i32				viewport_x, viewport_y;
		u32				viewport_width, viewport_height;

		/* This only holds valid information during drawAll()-calls */
		FrameInfo_t		drawInfo;

};


#endif /* __RC_WORLD_H__ */

